@@ -69,6 +69,7 @@ class DpCloudServer(Machine):
             return job.upload_path
         else:
             program_id = self.context.remote_profile.get('program_id')
+            program_id = self.context.remote_profile.get('project_id', program_id)
             if program_id is None:
                 dlog.info("can not find program id in remote profile, upload to default program id.")
                 program_id = 0
@@ -89,13 +90,15 @@ class DpCloudServer(Machine):
         input_data['job_resources'] = job_resources
         input_data['command'] = f"bash {job.script_file_name}"
         # input_data['backward_files'] = self._gen_backward_files_list(job)
-        if self.context.remote_profile.get('program_id') is None:
+        program_id = self.context.remote_profile.get('program_id')
+        program_id = self.context.remote_profile.get('project_id', program_id)
+        if program_id is None:
             warnings.warn('program_id is compulsory.')
         job_id, group_id = self.api.job_create(
             job_type=input_data['job_type'],
             oss_path=input_data['job_resources'],
             input_data=input_data,
-            program_id=self.context.remote_profile.get('program_id', None),
+            program_id=program_id,
             group_id=self.group_id
         )
         if self.grouped:
@@ -121,7 +124,14 @@ class DpCloudServer(Machine):
         check_return = None
         # print("api",self.api_version,self.input_data.get('job_group_id'),job.job_id)
         check_return = self.api.get_tasks(job_id,group_id)
-        try:
+        assert (check_return is not None), f"Failed to retrieve tasks information. To resubmit this job, please " \
+                                           f"try again, if this problem still exists please delete the submission " \
+                                           f"file and try again.\nYou can check submission.submission_hash in the " \
+                                           f"previous log or type `grep -rl \"{job_id}:job_group_id:{group_id}\" " \
+                                           f"~/.dpdispatcher/dp_cloud_server/` to find corresponding file. " \
+                                           f"You can try with command:\n    " \
+                                           f'rm $(grep -rl "{job_id}:job_group_id:{group_id}" ~/.dpdispatcher/dp_cloud_server/)'
+        try:            
             dp_job_status = check_return["status"]
         except IndexError as e:
             dlog.error(f"cannot find job information in bohrium for job {job.job_id}. check_return:{check_return}; retry one more time after 60 seconds")
