@@ -1,5 +1,5 @@
 from dpdispatcher.base_context import BaseContext
-import os,shutil,hashlib
+import os,shutil,hashlib,signal
 import subprocess as sp
 from glob import glob
 from dpdispatcher import dlog
@@ -230,7 +230,11 @@ class LocalContext(BaseContext) :
                         pass
                     elif (os.path.exists(rfile)) and (not os.path.exists(lfile)) :
                         # trivial case, download happily
-                        shutil.move(rfile, lfile)
+                        # for links, copy instead of moving (default behavior of copyfile is following symlinks)
+                        if not os.path.islink(rfile):
+                            shutil.move(rfile, lfile)
+                        else:
+                            shutil.copyfile(rfile, lfile)
                     elif (os.path.exists(rfile)) and (os.path.exists(lfile)) :
                         # both exists, replace!
                         dlog.info('find existing %s, replacing by %s' % (lfile, rfile))
@@ -276,7 +280,10 @@ class LocalContext(BaseContext) :
                     pass
                 elif (os.path.exists(rfile)) and (not os.path.exists(lfile)) :
                     # trivial case, download happily
-                    shutil.move(rfile, lfile)
+                    if not os.path.islink(rfile):
+                        shutil.move(rfile, lfile)
+                    else:
+                        shutil.copyfile(rfile, lfile)
                 elif (os.path.exists(rfile)) and (os.path.exists(lfile)) :
                     dlog.info(f"both exist rfile:{rfile}; lfile:{lfile}")
                     # both exists, replace!
@@ -397,8 +404,8 @@ class LocalContext(BaseContext) :
         os.chdir(cwd)        
         return proc
 
-    def kill(self, proc):
-        proc.kill()
+    def kill(self, job_id):
+        os.kill(job_id, signal.SIGTERM)
 
     def check_finish(self, proc):
         return (proc.poll() != None)
