@@ -5,6 +5,7 @@ from dpdispatcher import dlog
 from dpdispatcher.JobStatus import JobStatus
 from typing import List
 from dargs import Argument
+from dpdispatcher.utils import retry, RetrySignal
 
 
 lsf_script_header_template = """\
@@ -64,7 +65,8 @@ class LSF(Machine):
 
         return lsf_script_header
 
-    def do_submit(self, job, retry=0, max_retry=3):
+    @retry()
+    def do_submit(self, job):
         script_file_name = job.script_file_name
         script_str = self.gen_script(job)
         job_id_name = job.job_hash + '_job_id'
@@ -75,13 +77,7 @@ class LSF(Machine):
                 'cd %s && %s %s' % (self.context.remote_root, 'bsub < ', script_file_name)
             )
         except RuntimeError as err:
-            if retry < max_retry:
-                dlog.warning(err)
-                dlog.warning("Sleep 60 s and retry submitting...")
-                # rest 60s
-                time.sleep(60)
-                return self.do_submit(job, retry=retry+1, max_retry=max_retry)
-            raise
+            raise RetrySignal(err) from err
 
         subret = (stdout.readlines())
         job_id = subret[0].split()[1][1:-1]
@@ -98,7 +94,8 @@ class LSF(Machine):
     def sub_script_head(self, res):
         pass
 
-    def check_status(self, job, retry=0, max_retry=3):
+    @retry()
+    def check_status(self, job):
         try:
             job_id = job.job_id
         except AttributeError:
@@ -115,15 +112,8 @@ class LSF(Machine):
                 return JobStatus.terminated
         elif ret != 0:
             # just retry when any unknown error raised.
-            if retry < max_retry:
-                dlog.warning("Get error code %d in checking status through ssh with job: %s . message: %s" %
+            raise RetrySignal("Get error code %d in checking status through ssh with job: %s . message: %s" %
                     (ret, job.job_hash, err_str))
-                dlog.warning("Sleep 60 s and retry checking...")
-                # rest 60s
-                time.sleep(60)
-                return self.check_status(job, retry=retry + 1, max_retry=max_retry)
-            raise RuntimeError("status command bjobs fails to execute.\n error info: %s \nreturn code %d\n"
-                               % (err_str, ret))
         status_out = stdout.read().decode('utf-8').split('\n')
         if len(status_out) < 2:
             return JobStatus.unknown
