@@ -11,8 +11,12 @@ from dpdispatcher import dlog
 from dargs.dargs import Argument
 from typing import List
 import pathlib
+import socket
 # from dpdispatcher.submission import Machine
-from dpdispatcher.utils import get_sha256, generate_totp, rsync
+from dpdispatcher.utils import (
+    get_sha256, generate_totp, rsync,
+    retry, RetrySignal,
+)
 
 class SSHSession (object):
     def __init__(self,
@@ -24,6 +28,7 @@ class SSHSession (object):
                 passphrase=None,
                 timeout=10,
                 totp_secret=None,
+                tar_compress=True
                 ):
 
         self.hostname = hostname
@@ -35,6 +40,7 @@ class SSHSession (object):
         self.timeout = timeout
         self.totp_secret = totp_secret
         self.ssh = None
+        self.tar_compress = tar_compress
         self._setup_ssh()
 
     # @classmethod
@@ -106,19 +112,97 @@ class SSHSession (object):
         # machine = self.machine
         self.ssh = paramiko.SSHClient()
         self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy)
-        if self.totp_secret and self.password is None:
-            self.password = generate_totp(self.totp_secret)
-        self.ssh.connect(hostname=self.hostname, port=self.port,
-                        username=self.username, password=self.password,
-                        key_filename=self.key_filename, timeout=self.timeout,passphrase=self.passphrase,
-                        compress=True,
-                        )
-        assert(self.ssh.get_transport().is_active())
-        transport = self.ssh.get_transport()
-        transport.set_keepalive(60)
+        # if self.totp_secret and self.password is None:
+        #     self.password = generate_totp(self.totp_secret)
+        # self.ssh.connect(hostname=self.hostname, port=self.port,
+        #                 username=self.username, password=self.password,
+        #                 key_filename=self.key_filename, timeout=self.timeout,passphrase=self.passphrase,
+        #                 compress=True,
+        #                 )
+        # assert(self.ssh.get_transport().is_active())
+        # transport = self.ssh.get_transport()
+        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        sock.settimeout(self.timeout)
+        sock.connect((self.hostname, self.port))
+
+        #Make a Paramiko Transport object using the socket
+        ts = paramiko.Transport(sock)
+        ts.banner_timeout = 60
+        ts.use_compression(compress=True)
+
+        #Tell Paramiko that the Transport is going to be used as a client
+        ts.start_client(timeout=self.timeout)
+
+        #Begin authentication; note that the username and callback are passed
+        if self.totp_secret:
+            ts.auth_interactive(self.username, self.inter_handler)
+        else:
+            key_path = os.path.join(os.path.expanduser("~"), ".ssh", "id_rsa")
+            if self.key_filename:
+                key_path = os.path.abspath(self.key_filename)
+            key = None
+            if os.path.exists(key_path):
+                try:
+                    key = paramiko.RSAKey.from_private_key_file(key_path)
+                except paramiko.PasswordRequiredException:
+                    key = paramiko.RSAKey.from_private_key_file(key_path, self.passphrase)
+            if key:
+                try:
+                    ts.auth_publickey(self.username, key)
+                except paramiko.ssh_exception.AuthenticationException:
+                    if self.password:
+                        ts.auth_password(self.username, self.password)
+                    else:
+                        raise RuntimeError("Authentication failed, try to provide password")
+            elif self.password:
+                ts.auth_password(self.username, self.password)
+            else:
+                raise RuntimeError("Please provide at least one form of authentication")
+        assert(ts.is_active())
+        #Opening a session creates a channel along the socket to the server
+        ts.open_session(timeout=self.timeout)
+        ts.set_keepalive(60)
+        self.ssh._transport = ts
         # reset sftp
         self._sftp = None
                         
+    def inter_handler(self, title, instructions, prompt_list):
+        """
+        inter_handler: the callback for paramiko.transport.auth_interactive
+
+        The prototype for this function is defined by Paramiko, so all of the
+        arguments need to be there, even though we don't use 'title' or
+        'instructions'.
+
+        The function is expected to return a tuple of data containing the
+        responses to the provided prompts. Experimental results suggests that
+        there will be one call of this function per prompt, but the mechanism
+        allows for multiple prompts to be sent at once, so it's best to assume
+        that that can happen.
+
+        Since tuples can't really be built on the fly, the responses are 
+        collected in a list which is then converted to a tuple when it's time
+        to return a value.
+
+        Experiments suggest that the username prompt never happens. This makes
+        sense, but the Username prompt is included here just in case.
+        """
+
+        resp = []  #Initialize the response container
+
+        #Walk the list of prompts that the server sent that we need to answer
+        for pr in prompt_list:
+            #str() used to to make sure that we're dealing with a string rather than a unicode string
+            #strip() used to get rid of any padding spaces sent by the server
+            pr_str = str(pr[0]).strip().lower()
+            if "username" in pr_str:
+                resp.append(self.username)
+            elif "password" in pr_str:
+                resp.append(self.password)
+            elif "verification" in pr_str and self.totp_secret:
+                resp.append(generate_totp(self.totp_secret))
+
+        return tuple(resp)  #Convert the response list to a tuple and return it
 
     def get_ssh_client(self) :
         return self.ssh
@@ -129,19 +213,17 @@ class SSHSession (object):
     def close(self) :
         self.ssh.close()
 
-    def exec_command(self, cmd, retry = 0):
+    @retry(sleep=1)
+    def exec_command(self, cmd):
         """Calling self.ssh.exec_command but has an exception check."""
         try:
             return self.ssh.exec_command(cmd)
-        except paramiko.ssh_exception.SSHException:
+        except (paramiko.ssh_exception.SSHException, socket.timeout) as e:
             # SSH session not active
             # retry for up to 3 times
-            if retry < 3:
-                dlog.warning("SSH session not active in calling %s, retry the command..." % cmd)
-                # ensure alive
-                self.ensure_alive()
-                return self.exec_command(cmd, retry = retry+1)
-            raise RuntimeError("SSH session not active")
+            # ensure alive
+            self.ensure_alive()
+            raise RetrySignal("SSH session not active in calling %s" % cmd) from e
 
     @property
     def sftp(self):
@@ -164,6 +246,7 @@ class SSHSession (object):
         doc_timeout = 'timeout of ssh connection'
         doc_totp_secret = 'Time-based one time password secret. It should be a base32-encoded string' \
                           ' extracted from the 2D code.'
+        doc_tar_compress = 'The archive will be compressed in upload and download if it is True. If not, compression will be skipped.'
 
         ssh_remote_profile_args = [
             Argument("hostname", str, optional=False, doc=doc_hostname),
@@ -174,6 +257,7 @@ class SSHSession (object):
             Argument("passphrase", [str, None], optional=True, default=None, doc=doc_passphrase),
             Argument("timeout", int, optional=True, default=10, doc=doc_timeout),
             Argument("totp_secret", str, optional=True, default=None, doc=doc_totp_secret),
+            Argument("tar_compress", bool, optional=True, default=True, doc = doc_tar_compress),
         ]
         ssh_remote_profile_format = Argument("ssh_session", dict, ssh_remote_profile_args)
         return ssh_remote_profile_format
@@ -244,6 +328,7 @@ class SSHContext(BaseContext):
         #     key_filename = jdata.get('key_filename', None),
         #     passphrase = jdata.get('passphrase', None),
         #     timeout = jdata.get('timeout', 10),
+        #     tar_compress = jdata.get('tar_compress', True)
         # )
         local_root = context_dict['local_root']
         remote_root = context_dict['remote_root']
@@ -376,7 +461,7 @@ class SSHContext(BaseContext):
             # convert to relative path to local_root
             file_list = [os.path.relpath(jj, self.local_root) for jj in file_list] 
 
-        self._put_files(file_list, dereference = dereference, directories=directory_list)
+        self._put_files(file_list, dereference = dereference, directories=directory_list, tar_compress = self.remote_profile.get('tar_compress', None))
         os.chdir(cwd)
 
     def download(self, 
@@ -408,7 +493,7 @@ class SSHContext(BaseContext):
                 file_list.extend(errors)
         file_list.extend(submission.backward_common_files)
         if len(file_list) > 0:
-            self._get_files(file_list)
+            self._get_files(file_list, tar_compress = self.remote_profile.get('tar_compress', None))
         os.chdir(cwd)
         
     def block_checkcall(self, 
@@ -448,8 +533,13 @@ class SSHContext(BaseContext):
 
     def write_file(self, fname, write_str):
         self.ssh_session.ensure_alive()
-        with self.sftp.open(pathlib.PurePath(os.path.join(self.remote_root, fname)).as_posix(), 'w') as fp :
+        fname = pathlib.PurePath(os.path.join(self.remote_root, fname)).as_posix()
+        # to prevent old file from being overwritten but cancelled, create a temporary file first
+        # when it is fully written, rename it to the original file name
+        with self.sftp.open(fname + "~", 'w') as fp :
             fp.write(write_str)
+        # sftp.rename may throw OSError
+        self.block_checkcall("mv %s %s" % (fname + "~", fname))
 
     def read_file(self, fname):
         self.ssh_session.ensure_alive()
@@ -506,6 +596,7 @@ class SSHContext(BaseContext):
                    files,
                    dereference = True,
                    directories = None,
+                   tar_compress = True,
                    ) :
         """Upload files to server.
 
@@ -520,21 +611,31 @@ class SSHContext(BaseContext):
         directories: list, default: None
             uploaded directories non-recursively. Use `files` for uploading
             recursively
+        tar_compress: bool, default: True
+            If tar_compress is True, compress the archive using gzip
+            It it is False, then it is uncompressed
         """
-        of = self.submission.submission_hash + '.tgz'
+        of_suffix = '.tgz'
+        tarfile_mode = "w:gz"
+        kwargs = {'compresslevel': 6}
+        if not tar_compress :
+            of_suffix = '.tar'
+            tarfile_mode = "w"
+            kwargs = {}
+                    
+        of = self.submission.submission_hash + of_suffix
         # local tar
         cwd = os.getcwd()
         os.chdir(self.local_root)
         if os.path.isfile(of) :
             os.remove(of)
-        with tarfile.open(of, "w:gz", dereference = dereference, compresslevel=6) as tar:
+        with tarfile.open(of, tarfile_mode, dereference = dereference, **kwargs) as tar:
             for ii in files :
                 tar.add(ii)
             if directories is not None:
                 for ii in directories:
                     tar.add(ii, recursive=False)
         os.chdir(cwd)
-
         self.ssh_session.ensure_alive()
         try:
             self.sftp.mkdir(self.remote_root)
@@ -554,19 +655,29 @@ class SSHContext(BaseContext):
         self.sftp.remove(to_f)
 
     def _get_files(self, 
-                   files) :
-        of = self.submission.submission_hash + '.tar.gz'
+                   files,
+                   tar_compress = True) :
+        
+        of_suffix = '.tar.gz'
+        tarfile_mode = "r:gz"
+        tar_command = 'czfh'
+        if not tar_compress :
+            of_suffix = '.tar'
+            tarfile_mode = "r"
+            tar_command = 'cfh'
+
+        of = self.submission.submission_hash + of_suffix
         # remote tar
         # If the number of files are large, we may get "Argument list too long" error.
         # Thus, "-T" accepts a file containing the list of files
         per_nfile = 100
         ntar = len(files) // per_nfile + 1
         if ntar <= 1:
-            self.block_checkcall('tar czfh %s %s' % (of, " ".join(files)))
+            self.block_checkcall('tar %s %s %s' % (tar_command, of, " ".join(files)))
         else:
             file_list_file = os.path.join(self.remote_root, ".tmp.tar." + str(uuid.uuid4()))
             self.write_file(file_list_file, "\n".join(files))
-            self.block_checkcall('tar czfh %s -T %s' % (of, file_list_file))
+            self.block_checkcall('tar %s %s -T %s' % (tar_command, of, file_list_file))
         # trans
         from_f = pathlib.PurePath(os.path.join(self.remote_root, of)).as_posix()
         to_f = pathlib.PurePath(os.path.join(self.local_root, of)).as_posix()
@@ -576,9 +687,9 @@ class SSHContext(BaseContext):
         # extract
         cwd = os.getcwd()
         os.chdir(self.local_root)
-        with tarfile.open(of, "r:gz") as tar:
+        with tarfile.open(of, mode = tarfile_mode) as tar:
             tar.extractall()
-        os.chdir(cwd)        
+        os.chdir(cwd)    
         # cleanup
         os.remove(to_f)
         self.sftp.remove(from_f)
